require 'spec_helper'


describe OO::CheckLicence do

  describe '.new(attrs = {})' do
    let(:service_object) { described_class.new(attrs) }
    let(:licence_key) { 'glass.take' }

    subject { service_object }

    context 'when required attrs are missing' do
      context 'when :key (checked licence key) is missing' do
        let(:attrs) { { in: [] } }
        it { expect { subject }.to raise_error(KeyError) }
      end

      context 'when :in (licence keys list) is missing' do
        let(:attrs) { { key: licence_key } }
        it { expect { subject }.to raise_error(KeyError) }
      end
    end

    context 'when all required attrs were passed' do
      let(:attrs) { { key: licence_key, in: [] } }

      it { is_expected.to be_a(described_class) }

      describe 'builds a licence' do
        let(:licence_class) { OO::Licence }

        describe '@licence' do
          let(:licence) { service_object.instance_variable_get(:@licence) }

          subject { licence }
          it { is_expected.to be_a(licence_class) }

          describe '#key' do
            subject { licence.key }
            it { is_expected.to eq(licence_key) }
          end
        end
      end
    end
  end

  describe '#call' do
    let(:service_object) { described_class.new(key: licence_key, in: licence_keys) }
    let(:licence) { OO::Licence.new(key: licence_key) }
    let(:licence_key) { 'glass.take' }

    subject { service_object.call }

    describe 'returns all keys wider or equal to given key found in given list' do
      context 'when licence key is included in licence keys list' do
        context 'and is an only match' do
          let(:licence_keys) { [licence_key] }
          it { is_expected.to eq([licence_key]) }
        end
      end

      context 'when exact licence key is missing in given list' do
        context 'but there are wider keys' do
          let(:licence_keys) { licence.genre_masks }
          it { is_expected.to eq(licence.genre_masks) }
        end
      end

      context 'when licence key is missing in given list' do
        let(:licence_keys) { %w(a b c d) }
        it { is_expected.to eq([]) }

        context 'but found parent key' do
          let(:licence_keys) { [licence.parent.key] }
          it { is_expected.to eq([]) }
        end
      end
    end
  end
end
