require 'spec_helper'

describe OO::Configuration do
  describe '.new' do
    let(:configuration) { described_class.new }
    let(:default_licence_max_depth) { 4 }
    let(:default_licence_keys) { [] }
    let(:default_licence_missing_reaction) do
      -> { :licence_missing_reaction_not_configured }
    end

    let(:default_licence_checkers_default_user) do
      -> { :licence_checkers_default_user_not_configured }
    end

    let(:default_licence_checkers_default_key) do
      -> { :licence_checkers_default_key_not_configured }
    end

    subject { configuration }

    it { is_expected.to be_a(described_class) }
    describe 'it is set to default values' do
      describe 'licence_max_depth' do
        subject { configuration.licence_max_depth }
        it { is_expected.to eq(default_licence_max_depth) }
      end

      describe 'default_licence_keys' do
        subject { configuration.default_licence_keys }
        it { is_expected.to eq(default_licence_keys) }
      end

      describe 'licence_missing_reaction' do
        let(:licence_missing_reaction) { configuration.licence_missing_reaction }
        subject { licence_missing_reaction }
        it { is_expected.to be_a(Proc) }
        describe "#call" do
          subject { licence_missing_reaction.call }
          it { is_expected.to eq(:licence_missing_reaction_not_configured) }
        end
      end

      describe 'licence_checkers_default_user' do
        let(:licence_checkers_default_user) { configuration.licence_checkers_default_user }
        subject { licence_checkers_default_user }
        it { is_expected.to be_a(Proc) }
        describe "#call" do
          subject { licence_checkers_default_user.call }
          it { is_expected.to eq(:licence_checkers_default_user_not_configured) }
        end
      end

      describe 'licence_checkers_default_key' do
        let(:licence_checkers_default_key) { configuration.licence_checkers_default_key }
        subject { licence_checkers_default_key }
        it { is_expected.to be_a(Proc) }
        describe "#call" do
          subject { licence_checkers_default_key.call }
          it { is_expected.to eq(:licence_checkers_default_key_not_configured) }
        end
      end
    end

    describe '#licence_max_depth=(depth)' do
      context 'when depth is not a fixnum' do
        let(:depth) { 'a' }
        let(:error) { OO::Errors::LicenceMaxDepthInvalid }

        subject { -> { configuration.licence_max_depth = depth } }
        it { is_expected.to raise_error(error) }
      end

      context 'when depth is not greater than 0' do
        let(:depth) { 0 }
        let(:error) { OO::Errors::LicenceMaxDepthInvalid }

        subject { -> { configuration.licence_max_depth = depth } }
        it { is_expected.to raise_error(error) }
      end
    end

    describe '#licence_missing_reaction=(action)' do
      context 'when action is not a Proc object' do
        let(:action) { :invalid }
        let(:error) { OO::Errors::LicenceMissingReactionInvalid }

        subject { -> { configuration.licence_missing_reaction = action } }
        it { is_expected.to raise_error(error) }
      end
    end

    describe '#licence_checkers_default_user=(action)' do
      context 'when action is not a Proc object' do
        let(:action) { :invalid }
        let(:error) { OO::Errors::LicenceCheckersDefaultUserInvalid }

        subject { -> { configuration.licence_checkers_default_user = action } }
        it { is_expected.to raise_error(error) }
      end
    end

    describe '#licence_checkers_default_key=(action)' do
      context 'when action is not a Proc object' do
        let(:action) { :invalid }
        let(:error) { OO::Errors::LicenceCheckersDefaultKeyInvalid }

        subject { -> { configuration.licence_checkers_default_key = action } }
        it { is_expected.to raise_error(error) }
      end
    end

    describe '#default_licence_keys=(licence_keys)' do
      context 'when licence_keys is not an array' do
        let(:licence_keys) { :invalid }
        let(:error) { OO::Errors::DefaultLicenceKeysInvalid }

        subject { -> { configuration.default_licence_keys = licence_keys } }
        it { is_expected.to raise_error(error) }
      end
    end
  end
end
